@@ -90,6 +90,13 @@ def replaceTable(expr, basetable, parenttable, outertable=None):
 
 def canMoveIn(expr, inner, outer):
     
+    if inner.groupbys.getTables().filter(lambda x: x.leftjoin):
+        # return expr in inner.joincond.children
+        return expr.getTables() <= inner.baseTables()
+    
+    if expr in inner.joincond.children:
+        return True
+    
     # if it's only part of a single subquery
     willmove = outer.getTables().filter(lambda x: x.isQuery() and expr.getTables() ^ x.getTables()).len() <= 1
     
@@ -198,6 +205,9 @@ def moveInAll(inner, outer, debug=False):
 
 def canMoveOut(table, inner, outer):
     
+    if inner.groupbys.getTables().filter(lambda x: x.leftjoin):
+        return table in outer.baseTables() and table not in inner.baseTables()
+    
     # isisolated = inner.joincond._filter(table, getRerouteTables(outer)).getTables() <= L(table)
         
     # we ALWAYS want it IN unless the table appears in more than one subquery, in this case it's like we're "factorizing" it
@@ -251,7 +261,7 @@ def canReroute(expr, basetable, target, outer=None):
     # the target is the inner query
     # if the tables of expr are a subset of the tables of the target... and not aggregate and not constant
     # and we don't reroute exists exprs
-    isexists = type(expr) is FuncExistsExpr or (type(expr) is FuncExpr and expr.func.__name__ in ['notnull_','isnull_'])
+    isexists = expr.isExist() or (type(expr) is FuncExpr and expr.func.__name__ in ['notnull_','isnull_'])
     
     return basetable in expr.getTables() and expr.getTables() <= target.getTables() \
            and type(expr) is not ConstExpr and (not expr.isagg()) \
@@ -413,8 +423,7 @@ def canMerge(self, subquery=False):
     # the nonagg columns aren't aggs
     # OR it doesn't contain 
     # ie an expr that compares a table to an aggregate of that table
-    canmerge = self.allExprs()\
-                   .bind(lambda x: L(x) + x.descendants())\
+    canmerge = self.allDescendants()\
                    .filter(lambda x: isinstance(x, AggExpr))\
                    .bind(lambda x: x.baseExprs())\
                    .fmap(lambda x: x.getRef())\
@@ -587,11 +596,13 @@ def reduceQuery(self, debug=False, subquery=False):
         print(' # %% ^━━━━━━━━━━━━━━━━━ REDUCING... ━━━━━━━━━━━━━━━━━━━━━━━━^ ')
         print(self.sql(reduce=False, debug=True))        
     
-    # if the selects are left joined, then make the query left joined
-    if self.isQuery() and self.getTables().fmap(lambda x: x.leftjoin).all():
-        self.leftjoin = True
-        for table in self.getTables():
+    # if the groupbys are left joined, then make the query left joined
+    if self.isQuery() and self.groupbys.getTables().fmap(lambda x: x.leftjoin).all():
+        self.leftjoin = True        
+        for table in self.groupbys.getTables():
             table.leftjoin = False
+            for der in table.derivatives:
+                der.leftjoin = False
     
     # STEP 1: MERGE SUBQUERIES WITH THE SAME GROUP BYS
     selfcopy = copy(self)
@@ -624,7 +635,7 @@ def reduceQuery(self, debug=False, subquery=False):
     
     
     # STEP 3: MOVING OUT - everything that's grouped by or joined to something that's grouped by
-    for newtable in self.subQueries():        
+    for newtable in self.subQueries():
         moveOutAll(inner=newtable, outer=self, debug=debug)
     
     
@@ -668,9 +679,9 @@ def reduceQuery(self, debug=False, subquery=False):
             print(self.sql(reduce=False, debug=True))
             print(' # %% ^━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━^ ')
     
-    # # STEP 6: MOVE IN THE "HAVING" EXPRS
-    # for newtable in self.subQueries():
-    #     moveInHaving(inner=newtable, outer=self)
+    # STEP 6: MOVE IN THE "HAVING" EXPRS
+    for newtable in self.subQueries():
+        moveInHaving(inner=newtable, outer=self)
     
     
     # STEP 8: REDUCE CHILDREN
@@ -696,9 +707,9 @@ def reduceQuery(self, debug=False, subquery=False):
     
     # # STEP 10: REPEAT IF THERE'S STILL MORE TO DO
     # if getRerouteTables(self):
-        # if debug:
-        #     print("REDUCING AGAIN")
-        #     reduceQuery(self)
+    #     if debug:
+    #         print("REDUCING AGAIN")
+    #         reduceQuery(self)
     
     
     if debug:
